import { Form, Input, Button, Spin, Row, Col } from 'antd';
import { useEffect, useMemo, useState, useCallback } from 'react';
import { useNavigate } from 'react-router-dom';
import { respuestas } from '../../../utilities'
import HttpService from '../../../services/httpService';
import { useQuery, useModel } from '../../../hooks';
import { commonRules } from '../../../constants/rules';
import { Select } from "../../../components";

const endpoints = {
  municipio: "municipio",
};

const MunicipioDetalle = () => {
  const [form] = Form.useForm();
  const navigate = useNavigate();
  const [loading, setLoading] = useState(false);
  const query = useQuery();
  const id = query.get("id");
  const [estado, setEstado] = useState("");
  const [timer, setTimer] = useState(null);
  const [request, setRequest] = useState({})

  const estadoExtraParams = useMemo(() => ({
    idEstado: id,
  }), [id])

  const requestParams = useMemo(() => ({
    name: endpoints.municipio,
    id,
  }), [id])

  const { model, modelLoading } = useModel(request)

  const onFinish = async (values) => {
    try {
      setLoading(true);

      let body = {
        ...values,
      };

      if (id) {
        body.id = id
      }

      const res = await HttpService.post(`${endpoints.municipio}/guardar`, body);
      respuestas(res);
      if (res?.status === 200) {
        navigate('/administracion/catalogos/municipios')
      }
    } catch (error) {
      console.log(error);
      setLoading(false);
    } finally {
      setLoading(false);
    }
  }

  const onSearch = (value) => {
    clearTimeout(timer);
    const newTimer = setTimeout(() => {
      setEstado(value);
    }, 300);
    setTimer(newTimer);
  };


  useEffect(() => {
    if (id) {
      setRequest(requestParams)
    }
    return () => {
      setRequest({})
    }
  }, [id, requestParams])

  useEffect(() => {
    if (model) {
      form.setFieldsValue({ 
        ...model,
      })
    }
  }, [form, model])

  return (
    <Form
      layout="vertical"
      name="basic"
      form={form}
      onFinish={onFinish}
      onFinishFailed={() => {}}
    >
      <Row gutter={16}>
        <Col span={24}>
          <h2>Información del Municipio</h2>
        </Col>
        <Col md={8} xs={12}>
          <Form.Item
            label="Nombre"
            name="nombre"
            rules={[commonRules.requerido]}
          >
            <Input />
          </Form.Item>
        </Col>
        <Col md={8} xs={12}>
          <Form.Item
            label="Estado"
            name="idEstado"
            rules={[commonRules.requerido]}
          >
            <Select
              modelsParams={{ name: 'estado', ordenar: 'nombre' }}
              labelProp="nombre"
              valueProp="id"
              append={form.getFieldValue('idEstado') ? [{ id: form.getFieldValue('idEstado'), nombre: form.getFieldValue('estado') }] : []}
              onSearch={onSearch}
              extraParams={estadoExtraParams}
            />
          </Form.Item>
        </Col>
        <Col span={24}>
          <Form.Item>
            <Button
              type="primary"
              htmlType="submit"
              style={{ marginTop: "30px" }}
              loading={loading}
            >
              Guardar
            </Button>
          </Form.Item>
        </Col>
      </Row>
    </Form>
  );
};

export default MunicipioDetalle;
